@@ -8,20 +8,17 @@ package Petal;
 use Petal::Hash;
 use Petal::Cache::Disk;
 use Petal::Cache::Memory;
-use Petal::Parser;
+use Petal::Parser::XMLWrapper;
+use Petal::Parser::HTMLWrapper;
 use Petal::Canonicalizer::XML;
 use Petal::Canonicalizer::XHTML;
 use Petal::Functions;
-use Petal::Entities;
-use Petal::Encode;
 use File::Spec;
+use strict;
+use warnings;
 use Carp;
 use Safe;
 use Data::Dumper;
-use Scalar::Util;
-use strict;
-use warnings;
-use MKDoc::XML::Decode;
 
 
 # these are used as local variables when the XML::Parser
@@ -45,9 +42,9 @@ our $WARN_UNINIT = 0;
 # What do we use to parse input?
 our $INPUT  = 'XML';
 our $INPUTS = {
-    'XML'     => 'Petal::Parser',
-    'HTML'    => 'Petal::Parser',
-    'XHTML'   => 'Petal::Parser',
+    'XML'   => 'Petal::Parser::XMLWrapper',
+    'HTML'  => 'Petal::Parser::HTMLWrapper',
+    'XHTML' => 'Petal::Parser::HTMLWrapper',
 };
 
 
@@ -87,7 +84,7 @@ our $CURRENT_INCLUDES = 0;
 
 
 # this is for CPAN
-our $VERSION = '1.10_09';
+our $VERSION = '1.01';
 
 
 # The CodeGenerator class backend to use.
@@ -107,10 +104,6 @@ our $NS_URI = 'http://purl.org/petal/1.0/';
 our $XI_NS = 'xi';
 our $XI_NS_URI = 'http://www.w3.org/2001/XInclude';
 
-our $MT_NS       = 'metal';
-our $MT_NS_URI   = 'http://xml.zope.org/namespaces/metal';
-our $MT_NAME_CUR = 'main';
-
 
 # Displays the canonical template for template.xml.
 # You can set $INPUT using by setting the PETAL_INPUT environment variable.
@@ -151,15 +144,14 @@ sub main::lcode
 
 sub load_code_generator
 {
-    if (not $CodeGeneratorLoaded)
-    {
-	eval "require $CodeGenerator";
-	confess "Failed to load $CodeGenerator, $@" if $@;
-	$CodeGeneratorLoaded = 1;
-    }
+	if (not $CodeGeneratorLoaded)
+	{
+	    eval "require $CodeGenerator";
+	    confess "Failed to load $CodeGenerator, $@" if $@;
+	    $CodeGeneratorLoaded = 1;
+	}
 }
 
-
 # Instanciates a new Petal object.
 sub new
 {
@@ -177,32 +169,15 @@ sub new
 # if the language has been specified, let's try to
 # find which template we can use.
 sub _initialize
-{
-    my $self  = shift;
-    my $file  = $self->{file};
-    if ($file =~ /#/)
-    {
-	my ($file, $macro) = split /#/, $file, 2;
-	$self->{file}  = $file;
-	$self->_initialize_lang();
-	$self->{file} .= "#$macro";
-    }
-    else
-    {
-	$self->_initialize_lang();
-    }
-}
-
-
-sub _initialize_lang
 {
     my $self = shift;
     my $lang = $self->language() || return;
-    my @dirs = $self->base_dir();
-    @dirs    = map { File::Spec->canonpath ("$_/$self->{file}") } @dirs;
     
-    $self->{file}  =~ s/\/$//;
-    my $filename   = Petal::Functions::find_filename ($lang, @dirs);
+    my @dirs = $self->base_dir();
+    @dirs = map { File::Spec->canonpath ("$_/$self->{file}") } @dirs;
+    
+    $self->{file} =~ s/\/$//;
+    my $filename = Petal::Functions::find_filename ($lang, @dirs);
     $self->{file} .= "/$filename" if ($filename);
 }
 
@@ -254,6 +229,7 @@ sub _base_dir
 }
 
 
+
 # _include_compute_path ($path);
 # ------------------------------
 # Computes the new absolute path from the current
@@ -262,13 +238,6 @@ sub _include_compute_path
 {
     my $self  = shift;
     my $file  = shift;
-    
-    # this is for metal self-includes
-    if ($file =~ /^#/)
-    {
-	$file = $self->{file} . $file;
-    }
-    
     return $file unless ($file =~ /^\./);
     
     my $path = $self->{file};
@@ -315,10 +284,10 @@ sub _include_compute_path
 sub process
 {
     my $self = shift;
-    $self->_process_absolutize_pathes();
     
-    # ok, from there on we need to override any global variable with stuff
-    # that might have been specified when constructing the object
+    # ok, from there on we need to override any global
+    # variable with stuff that might have been specified
+    # when constructing the object
     local $TAINT              = defined $self->{taint}              ? $self->{taint}              : $TAINT;
     local $ERROR_ON_UNDEF_VAR = defined $self->{error_on_undef_var} ? $self->{error_on_undef_var} : $ERROR_ON_UNDEF_VAR;
     local $DISK_CACHE         = defined $self->{disk_cache}         ? $self->{disk_cache}         : $DISK_CACHE;
@@ -327,7 +296,7 @@ sub process
     local $INPUT              = defined $self->{input}              ? $self->{input}              : $INPUT;
     local $OUTPUT             = defined $self->{output}             ? $self->{output}             : $OUTPUT;
     local $BASE_DIR           = defined $self->{base_dir} ? do { ref $self->{base_dir} ? undef : $self->{base_dir} } : $BASE_DIR;
-    local @BASE_DIR           = defined $self->{base_dir} ? do { ref $self->{base_dir} ? @{$self->{base_dir}} : () } : @BASE_DIR;
+    local @BASE_DIR           = defined $self->{base_dir} ? do { ref $self->{base_dir} ? @{$self->{base_dir}} : undef } : @BASE_DIR;
     local $LANGUAGE           = defined $self->{default_language}   ? $self->{default_language}   : $LANGUAGE;
     local $DEBUG_DUMP         = defined $self->{debug_dump}         ? $self->{debug_dump}         : $DEBUG_DUMP;
     local $DECODE_CHARSET     = defined $self->{decode_charset}     ? $self->{decode_charset}     : $DECODE_CHARSET;
@@ -346,12 +315,11 @@ sub process
 	else                            { $hash = new Petal::Hash (@_)         }
 	
 	my $coderef = $self->_code_memory_cached;
-	die "\$coderef is undefined\n\n" unless $coderef;
-	die "\$hash is undefined\n\n" unless $hash;
 	$res = $coderef->($hash);
 	
 	$Petal::ENCODE_CHARSET and do {
-	    $res = Petal::Encode::p_encode ($Petal::ENCODE_CHARSET, $res);
+	    require "Encode.pm";
+	    $res = Encode::encode ($Petal::ENCODE_CHARSET, $res);
 	};
     };
     
@@ -360,43 +328,6 @@ sub process
 }
 
 
-# File::Spec->rel2abs() is pretty slow since it uses Cwd which does a
-# super-ugly backtick. Hence this method absolutizes base directories
-# only once. It is necessary to work with absolute base directories to
-# avoid cache conflicts.
-sub _process_absolutize_pathes
-{
-    my $self = shift;
-    
-    if (defined $BASE_DIR)
-    {
-	$BASE_DIR = File::Spec->rel2abs ($BASE_DIR) unless (
-	    File::Spec->file_name_is_absolute ($BASE_DIR)
-	     );
-    }
-    
-    @BASE_DIR = ( map { File::Spec->file_name_is_absolute ($_) ? $_ : File::Spec->rel2abs ($_) }
-		  map { defined $_ ? $_ : () } @BASE_DIR );
-    
-    if (defined $self->{base_dir})
-    {
-	if (ref $self->{base_dir})
-	{
-	    $self->{base_dir} = [
-		map { File::Spec->file_name_is_absolute ($_) ? $_ : File::Spec->rel2abs ($_) }
-		map { defined $_ ? $_ : () } @{$self->{base_dir}}
-	       ] if (defined $self->{base_dir});
-	}
-	else
-	{
-	    $self->{base_dir} = File::Spec->rel2abs ($self->{base_dir}) unless (
-		File::Spec->file_name_is_absolute ($self->{base_dir})
-		 );
-	}
-    }
-}
-
-
 sub _handle_error
 {
     my $self = shift;
@@ -413,7 +344,7 @@ sub _handle_error
 	ref $error and do {
 	    print ERROR "=============\n";
 	};
-	print ERROR "\n";
+	print "\n";
 	
 	print ERROR "Petal object dump:\n";
 	print ERROR "==================\n";
@@ -455,16 +386,16 @@ sub _code_with_line_numbers
     # add line numbers
     my $count = 0;
     @lines = map {
-	my $cur_line = $_;
-	$count++;
-	
-	# space padding so the line numbers nicely line up with each other
-	my $line_num = sprintf ("%" . length(scalar(@lines)) . "d", $count);
-	
-	# put line number and line back together
-	"${line_num}. ${cur_line}";
+      my $cur_line = $_;
+      $count++;
+
+      # space padding so the line numbers nicely line up with each other
+      my $line_num = sprintf("%" . length(scalar(@lines)) . "d", $count);
+
+      # put line number and line back together
+      "${line_num}. ${cur_line}";
     } @lines;
-    
+
     return join("\n", @lines);
 }
 
@@ -481,25 +412,6 @@ sub _file
 }
 
 
-sub _macro
-{
-    my $self = shift;
-    my $file = $self->_file;
-    $file =~ s/^.*#// || return;
-    return $file;
-}
-
-
-sub _file_path_with_macro
-{
-    my $self  = shift;
-    my $file  = $self->_file_path;
-    my $macro = $self->_macro;
-    my $res   = $macro ? "$file#$macro" : $file;
-    return $res;
-}
-
-
 # $self->_file_path;
 # ------------------
 #   computes the file of the absolute path where the template
@@ -508,15 +420,13 @@ sub _file_path
 {
     my $self = shift;
     my $file = $self->_file;
-    $file =~ s/#.*$//;
     my @dirs = $self->base_dir;
     
     foreach my $dir (@dirs)
     {
-	# my $base_dir  = File::Spec->canonpath ($dir);
-	# $base_dir     = File::Spec->rel2abs ($base_dir) unless ($base_dir =~ /^\//);
-	my $base_dir  = $dir;
-	$base_dir     =~ s/\/$//;
+	my $base_dir = File::Spec->canonpath ($dir);
+	$base_dir = File::Spec->rel2abs ($base_dir) unless ($base_dir =~ /^\//);
+	$base_dir =~ s/\/$//;
 	my $file_path = File::Spec->canonpath ($base_dir . '/' . $file);
 	return $file_path if (-e $file_path and -r $file_path);
     }
@@ -527,13 +437,12 @@ sub _file_path
 
 # $self->_file_data_ref;
 # ----------------------
-# slurps the template data into a variable and returns a
-# reference to that variable
+#   slurps the template data into a variable and returns a
+#   reference to that variable
 sub _file_data_ref
 {
     my $self      = shift;
     my $file_path = $self->_file_path;
-    $file_path =~ s/#.*$//;
     
     use bytes;
     open FP, "<$file_path" || die 'Cannot read-open $file_path';
@@ -542,72 +451,68 @@ sub _file_data_ref
     no bytes;
     
     $Petal::DECODE_CHARSET and do {
-	$res = Petal::Encode::p_decode ($Petal::DECODE_CHARSET, $res);
+	require "Encode.pm";
+	$res = Encode::decode ($Petal::DECODE_CHARSET, $res);
     };
     
     # kill template comments
     $res =~ s/\<!--\?.*?\-->//gsm;
-    
-    my $decode = ($OUTPUT =~ /HTML$/i or $INPUT =~ /HTML$/i) ?
-        new MKDoc::XML::Decode ('numeric', 'xhtml') :
-	new MKDoc::XML::Decode ('numeric');
-    
-    Petal::Encode->p_utf8_on ($res);
-    $res = $decode->process ($res);
-    Petal::Encode->p_utf8_off ($res);
-    
     return \$res;
 }
 
 
 # $self->_code_disk_cached;
 # -------------------------
-# Returns the Perl code data, using the disk cache if possible
+#   Returns the Perl code data, using the disk cache if
+#   possible
 sub _code_disk_cached
 {
     my $self = shift;
-    my $code = (defined $DISK_CACHE and $DISK_CACHE) ? Petal::Cache::Disk->get ($self->_file_path_with_macro) : undef;
+    my $file = $self->_file_path;
+    my $code = (defined $DISK_CACHE and $DISK_CACHE) ? Petal::Cache::Disk->get ($file) : undef;
     unless (defined $code)
     {
-	my $macro = $self->_macro() || $MT_NAME_CUR;
-	
-	local ($MT_NAME_CUR);
-	$MT_NAME_CUR = $macro;
-	
-	my $data_ref = $self->_canonicalize;
+	my $data_ref = $self->_file_data_ref;
+	$data_ref    = $self->_canonicalize;
+
 	load_code_generator();
 	$code = $CodeGenerator->process ($data_ref, $self);
-	Petal::Cache::Disk->set ($self->_file_path_with_macro, $code) if (defined $DISK_CACHE and $DISK_CACHE);
+	Petal::Cache::Disk->set ($file, $code) if (defined $DISK_CACHE and $DISK_CACHE);
     }
-    
     return $code;
 }
 
 
 # $self->_code_memory_cached;
 # ---------------------------
-# Returns the Perl code data, using the disk cache if possible
+#   Returns the Perl code data, using the disk cache if
+#   possible
 sub _code_memory_cached
 {
     my $self = shift;
-    my $code = (defined $MEMORY_CACHE and $MEMORY_CACHE) ? Petal::Cache::Memory->get ($self->_file_path_with_macro) : undef;
+    my $file = $self->_file_path;
+    my $code = (defined $MEMORY_CACHE and $MEMORY_CACHE) ? Petal::Cache::Memory->get ($file) : undef;
     unless (defined $code)
     {
 	my $code_perl = $self->_code_disk_cached;
-	my $VAR1 = undef;
+        my $VAR1 = undef;
 	
-	if ($TAINT)
+	if (0) # if ($TAINT) - doesn't work with repeat object
 	{
-		# important line, don't remove
-		($code_perl) = $code_perl =~ m/^(.+)$/s;
-		die "\$code_perl is empty after untainting!" unless defined $code_perl && $code_perl;
-		my $cpt = Safe->new ("Petal::CPT");
-		$cpt->permit ('entereval');
-		$cpt->permit ('leaveeval');
-		$cpt->permit ('require');
-		$code = $cpt->reval($code_perl);
-		confess ("Error in reval:\n" . $@ . "\n" . $self->_code_with_line_numbers) if $@;
-		warn "\$code is empty after reval.\n" . Dumper($code, $Petal::CPT::VAR1, length($code_perl)) unless $code;
+	    # important line, don't remove
+	    ($code_perl) = $code_perl =~ m/^(.+)$/s;
+	    my $cpt = Safe->new ("Petal::CPT");
+	    $cpt->permit ('entereval');
+	    $cpt->permit ('leaveeval');
+	    $cpt->permit ('require');
+	    $cpt->share_from ( 'Petal::Hash_Repeat', [ qw /$CUR $MAX/ ] );
+	    
+	    $cpt->reval($code_perl);
+	    confess ($@ . "\n" . $self->_code_with_line_numbers) if $@;
+	    
+	    # remove silly warning '"Petal::CPT::VAR1" used only once'
+	    $Petal::CPT::VAR1 if (0);
+	    $code = $Petal::CPT::VAR1;
 	}
 	else
 	{
@@ -616,16 +521,12 @@ sub _code_memory_cached
 	    $code = $VAR1;
 	}
 	
-	Petal::Cache::Memory->set ($self->_file_path_with_macro, $code) if (defined $MEMORY_CACHE and $MEMORY_CACHE);	
+        Petal::Cache::Memory->set ($file, $code) if (defined $MEMORY_CACHE and $MEMORY_CACHE);
     }
-    
     return $code;
 }
 
 
-
-
-
 # $self->_code_cache;
 # -------------------
 #   Returns TRUE if this object uses the code cache, FALSE otherwise
@@ -655,7 +556,6 @@ sub _canonicalize
 
 1;
 
-
 =head1 NAME
 
 Petal - Perl Template Attribute Language - TAL for Perl!
@@ -1294,81 +1194,6 @@ C<replace>, C<content>, C<omit-tag> and C<attributes> are NOT supported in
 conjunction with XIncludes.
 
 
-=head1 MORE INCLUDES: METAL
-
-Petal now supports a subset of the METAL specification, which is a very WYSIWYG
-compatible way of doing includes. The current implementation supports only two
-metal statements: define-macro and use-macro.
-
-
-=head2 define-macro
-
-In order to define a macro inside a file (i.e. a fragment to be included), you
-use the metal:use-macro directive. For example:
-
-  File foo.xml
-  ============
-
-  <html>
-    <body>
-      <p metal:define-macro="footer">
-        (c) Me (r)(tm) (pouet pouet)
-      </p>
-    </body>
-  </html>
-
-
-=head2 use-macro
-
-In order to use a previously defined macro, you use the metal:define-macro directive.
-For example:
-
-  File bar.xml
-  ============
-
-  <html>
-    <body>
-      ... plenty of content ...
-
-      <p metal:use-macro="foo.xml#footer">
-        Page Footer.
-      </p>
-    </body>
-  </html>
-
-
-=head2 self includes
-
-In Zope, METAL macros are expanded first, and then the TAL instructions are processed.
-However with Petal, METAL macros are expanded at run-time just like regular includes,
-which allows for recursive macros.
-
-This example templates a sitemap, which on a hierarchically organized site would
-be recursive by nature:
-
-  <html>
-    <body>
-      <p>Sitemap:</p>
-
-      <li metal:define-macro="recurse">
-        <a href="#"
-           petal:attributes="href child/Full_Path" 
-           petal:content="child/Title"
-        >Child Document Title</a>
-        <ul 
-          petal:define="children child/Children"
-          petal:condition="children"
-          petal:repeat="child children"
-        >
-          <li metal:use-macro="#recurse">Dummy Child 1</li>
-          <li petal:replace="nothing">Dummy Child 2</li>
-          <li petal:replace="nothing">Dummy Child 3</li>
-        </ul>
-      </li>
-    </body>
-  </html>
-
-
 =head1 EXPRESSIONS AND MODIFIERS
 
 Petal has the ability to bind template variables to the following Perl
