@@ -10,7 +10,6 @@ use strict;
 use warnings;
 use File::Spec;
 use Digest::MD5 qw /md5_hex/;
-use Petal::Encode;
 use Carp;
 
 
@@ -57,14 +56,10 @@ sub set
     my $data  = shift;
     my $key   = $class->compute_key ($file);
     my $tmp   = $class->tmp;
-    {
-	no utf8;
-	use bytes;
-	open FP, ">$tmp/$key" or
+    open FP, ">$tmp/$key" or
         ( Carp::cluck "Cannot write-open $tmp/$key" and return );
-	print FP $data;
-	close FP;
-    }
+    print FP $data;
+    close FP;
 }
 
 
@@ -127,7 +122,6 @@ sub current_mtime
 {
     my $class = shift;
     my $file = shift;
-    $file =~ s/#.*$//;
     my $mtime = (stat($file))[9];
     return $mtime;
 }
@@ -144,19 +138,15 @@ sub cached
     my $cached_filepath = $tmp . '/' . $key;
     
     (-e $cached_filepath) or return;
-
-    my $res = undef;
-    {
-	no utf8;
-	use bytes;
-	open FP, "<$tmp/$key" or
-            (Carp::cluck "Cannot read-open cached file for $tmp/$key" and return);
-	$res = join '', <FP>;
-	close FP;
-    }
+    
+    open FP, "<$tmp/$key" or
+        (Carp::cluck "Cannot read-open cached file for $tmp/$key" and return);
+    my $res = join '', <FP>;
+    close FP;
     
     $Petal::DECODE_CHARSET and do {
-	$res = Petal::Encode::p_decode ($Petal::DECODE_CHARSET, $res);
+	require "Encode.pm";
+	$res = Encode::decode ($Petal::DECODE_CHARSET, $res);
     };
     
     return $res;
