@@ -97,7 +97,7 @@ sub StartTag
 		$command =~ s/^\$//;
 		$command =~ s/^\{//;
 		$command =~ s/\}$//;
-		$command = $class->_encode_backslash_semicolon ($command);
+		$command = Petal::XML_Encode_Decode::encode_backslash_semicolon ($command);
 		$command = "<?var name=\"$command\"?>";
 		$text =~ s/\Q$var\E/$command/g;
 	    }
@@ -150,7 +150,7 @@ sub StartTag
 	{
 	    if (defined $att->{"$petal:omit-tag"})
 	    {
-		my $expression = $att->{"$petal:omit-tag"} || 'string:1';
+		my $expression = $att->{"$petal:omit-tag"};
 		$Petal::Canonicalizer::XML::NodeStack[$#Petal::Canonicalizer::XML::NodeStack]->{'omit-tag'} = $expression;
 		push @Petal::Canonicalizer::XML::Result, (defined $att_str and $att_str) ?
 		    "<?if name=\"false:$expression\"?><$tag $att_str /><?end?>" :
@@ -165,7 +165,7 @@ sub StartTag
 	{
 	    if (defined $att->{"$petal:omit-tag"})
 	    {
-		my $expression = $att->{"$petal:omit-tag"} || 'string:1';
+		my $expression = $att->{"$petal:omit-tag"};
 		$Petal::Canonicalizer::XML::NodeStack[$#Petal::Canonicalizer::XML::NodeStack]->{'omit-tag'} = $expression;
 		push @Petal::Canonicalizer::XML::Result, (defined $att_str and $att_str) ?
 		    "<?if name=\"false:$expression\"?><$tag $att_str><?end?>" :
