@@ -1,27 +1,3 @@
-package Petal::CodeGenerator::Decode;
-use strict;
-use warnings;
-use base qw /MKDoc::XML::Decode/;
-
-
-##
-# $class->process ($xml);
-# -----------------------
-# Expands the entities &apos; &quot; &gt; &lt; and &amp;
-# into their ascii equivalents.
-##
-sub process
-{
-    (@_ == 2) or warn "MKDoc::XML::Encode::process() should be called with two arguments";
-    
-    my $self = shift;
-    my $data = join '', @_;
-    $data    =~ s/&(#?[0-9A-Za-z]+)\\;/$self->entity_to_char ($1)/eg;
-    
-    return $data;
-}
-
-
 # ------------------------------------------------------------------
 # Petal::CodeGenerator - Generates Perl code from canonical syntax
 # ------------------------------------------------------------------
@@ -31,7 +7,7 @@ sub process
 # code that can be turned into a subroutine using eval().
 # ------------------------------------------------------------------
 package Petal::CodeGenerator;
-use MKDoc::XML::Decode;
+use Petal::XML_Encode_Decode;
 use strict;
 use warnings;
 use Carp;
@@ -112,7 +88,6 @@ sub code_header
     my $class = shift;
     $class->add_code("\$VAR1 = sub {");
     $class->indent_increment();
-    $class->add_code("require Petal::Encode;");
     $class->add_code("my \$hash = shift;");
     $class->add_code("my ".$class->_init_res.";");
     $class->add_code('local $^W = 0;') unless $Petal::WARN_UNINIT;
@@ -163,7 +138,7 @@ sub process
 	    %token_hash = (@atts1, @atts2); 
 	    foreach my $key (%token_hash)
 	    {
-		$token_hash{$key} = $class->_decode_backslash_semicolon ($token_hash{$key})
+		$token_hash{$key} = Petal::XML_Encode_Decode::decode_backslash_semicolon ($token_hash{$key})
 		    if (defined $token_hash{$key});
 	    }
 	    
@@ -195,12 +170,12 @@ sub process
 	}
 	else
 	{
-            my $string = quotemeta ($token);
-#            $string =~ s/\@/\\\@/gsm;
-#            $string =~ s/\$/\\\$/gsm;
-#            $string =~ s/\n/\\n/gsm;
-#            $string =~ s/\n//gsm;
-#            $string =~ s/\"/\\\"/gsm;
+            my $string = $token;
+            $string =~ s/\@/\\\@/gsm;
+            $string =~ s/\$/\\\$/gsm;
+            $string =~ s/\n/\\n/gsm;
+            $string =~ s/\n//gsm;
+            $string =~ s/\"/\\\"/gsm;
             $class->add_code($class->_add_res( '"' . $string . '";'));
         }
     }
@@ -227,9 +202,13 @@ sub _include
         $class->add_code ("my \$res = eval { Petal->new (file => '$path', lang => '$lang')->process (\$hash->new()) };") :
 	$class->add_code ("my \$res = eval { Petal->new ('$path')->process (\$hash->new()) };");
     
-    $class->add_code ("\$res = \"<!--\\n\$\@\\n-->\" if (defined \$\@ and \$\@);");
-    $class->add_code ("\$res = Petal::Encode::p_decode (\$Petal::ENCODE_CHARSET, \$res) if (\$Petal::ENCODE_CHARSET);");
-    $class->add_code ("Petal::Encode->p_utf8_on (\$res);");
+    $class->add_code ("\$res = \$\@ if (defined \$\@ and \$\@);");
+    $class->add_code ("if (scalar keys \%Encode::) {");
+    $class->indent_increment();
+    $class->add_code ("\$res = Encode::decode (\$Petal::ENCODE_CHARSET, \$res) if (\$Petal::ENCODE_CHARSET);");
+    $class->add_code ("Encode::_utf8_on (\$res) if (Encode->can ('_utf8_on'));");
+    $class->indent_decrement();
+    $class->add_code ("}");
     $class->add_code ("\$res;");
     $class->indent_decrement();
     $class->add_code ("} || '';");
@@ -247,19 +226,14 @@ sub _var
     
     (defined $variable and $variable) or
         confess "Cannot parse $token : 'name' attribute is not defined";
-    
+
     # set the variable in the $variables hash
     my $tmp = $variable;
     $tmp =~ s/\..*//;
     $variables->{$tmp} = 1;
     
     $variable =~ s/\'/\\\'/g;
-    $class->add_code ( $class->_add_res (('do {')) );
-    $class->indent_increment();
-    $class->add_code ('my $res = ' . $class->comp_expr ($variable) . ';');
-    $class->add_code ('(defined $res) ? $res : "";');
-    $class->indent_decrement();
-    $class->add_code ('};');
+    $class->add_code($class->_add_res($class->comp_expr($variable)." || '';"));
 }
 
 
@@ -405,42 +379,27 @@ sub _for
     {
 	$class->add_code("\@array = \@{".$class->comp_expr($variable)."};");
     }
-
-        
+    
+    $class->add_code ("local \$Petal::Hash_Repeat::MAX = \@array - 1;");
+    $class->add_code ("local \$Petal::Hash_Repeat::CUR = undef;");
     $class->add_code ("for (my \$i=0; \$i < \@array; \$i++) {");
     $class->indent_increment();
-    $class->add_code ("my \$hash   = \$hash->new();");
-    
-    # compute various might-be-useful variables
-    $class->add_code ("my \$number = \$i + 1;");
-    $class->add_code ("my \$odd    = \$number % 2;");
-    $class->add_code ("my \$even   = \$i % 2;");
-    $class->add_code ("my \$start  = (\$i == 0);");
-    $class->add_code ("my \$end    = (\$i == \$#array);");
-    $class->add_code ("my \$inner  = (\$i and \$i < \@array);");
-    
-    # backwards compatibility
-    $class->add_code ("\$hash->{__count__}    = \$number;");
-    $class->add_code ("\$hash->{__is_first__} = \$start;");
-    $class->add_code ("\$hash->{__is_last__}  = \$end;");
-    $class->add_code ("\$hash->{__is_inner__} = \$inner;");
-    $class->add_code ("\$hash->{__even__}     = \$even;");
-    $class->add_code ("\$hash->{__odd__}      = \$odd;");
-    
-    # new repeat style object
-    $class->add_code ("\$hash->{repeat} = {");
-    $class->indent_increment();
-    $class->add_code ("index  => \$i,");
-    $class->add_code ("number => \$number,");
-    $class->add_code ("even   => \$even,");
-    $class->add_code ("odd    => \$odd,");
-    $class->add_code ("start  => \$start,");
-    $class->add_code ("end    => \$end,");
-    $class->add_code ("inner  => \$inner,");
-    $class->add_code ("};");
-    $class->indent_decrement();
-    
+    $class->add_code ("\$hash->delete_cached (qr/__/);");
+    $class->add_code ("\$hash->delete_cached (qr/^\\Q$as\\E\$/);");
+    $class->add_code ("\$Petal::Hash_Repeat::CUR = \$i;");
+    $class->add_code ("\$hash->{__count__}    = sub { \$hash->{repeat}->number() };");
+    $class->add_code ("\$hash->{__is_first__} = sub { \$hash->{repeat}->start()  };");
+    $class->add_code ("\$hash->{__is_last__}  = sub { \$hash->{repeat}->end()    };");
+    $class->add_code ("\$hash->{__is_inner__} = sub { \$hash->{repeat}->inner()  };");
+    $class->add_code ("\$hash->{__even__}     = sub { \$hash->{repeat}->even()   };");
+    $class->add_code ("\$hash->{__odd__}      = sub { \$hash->{repeat}->odd()    };");
     $class->add_code ("\$hash->{'$as'} = \$array[\$i];");
+    
+#    $class->add_code("my \$hash = \$hash->new();");
+#    $class->add_code("my \$count= \$i + 1;");
+#    $class->add_code("\$hash->{__count__}    = \$count;");
+#    $class->add_code("\$hash->{__is_first__} = (\$count == 1);");
+#    $class->add_code("\$hash->{__is_last__}  = (\$count == \@array);");    
 }
 
 
@@ -467,14 +426,4 @@ sub _tokenize
 }
 
 
-sub _decode_backslash_semicolon
-{
-    my $class = shift;
-    my $data  = shift;
-    
-    my $decode = new Petal::CodeGenerator::Decode ('xml');
-    return $decode->process ($data);
-}
-
-
 1;
